import java.util.HashMap;
import java.util.Map;

public class LinkStatePacket {
    private int senderPort, originPort, sequenceNumber, timeToLive;
    private Map<Integer, Integer> adjacentDistances = new HashMap<>();

    // OLD l sender_port seq_number time_to_live adjacent_router_1_port-distance_1 ... adjacent_router_n_port-distance_n
    // l sender_port origin_port seq_number time_to_live adjacent_router_1_port-distance_1 ... adjacent_router_n_port-distance_n
    // Prof said it was okay if I added the origin_port to the lsp rather than hard code the sender port.
    public LinkStatePacket(String s){
        String[] tokens = s.split(" ");
        this.senderPort = Integer.parseInt(tokens[1]);
        this.originPort = Integer.parseInt(tokens[2]);
        this.sequenceNumber = Integer.parseInt(tokens[3]);
        this.timeToLive = Integer.parseInt(tokens[4]);
        for(int i = 5; i < tokens.length; i++){
            String[] adjacentDistance = tokens[i].split("-", 2);
            this.adjacentDistances.put(Integer.parseInt(adjacentDistance[0]), Integer.parseInt(adjacentDistance[1]));
        }
    }

    public LinkStatePacket(int senderPort, int originPort, int sequenceNumber, int timeToLive, Map<Integer, Integer> adjacents){
        this.originPort = originPort;
        this.senderPort = senderPort;
        this.sequenceNumber = sequenceNumber;
        this.timeToLive = timeToLive;
        this.adjacentDistances = adjacents;
    }

    public int getOriginPort(){
        return originPort;
    }

    public int getSenderPort(){
        return senderPort;
    }

    public void setSenderPort(int senderPort){
        this.senderPort = senderPort;
    }

    public int getSequenceNumber() {
        return sequenceNumber;
    }

    public int getTimeToLive(){
        return timeToLive;
    }

    public void decrementTTL(){
        timeToLive--;
    }

    public boolean isExpired(){
        return timeToLive <= 0;
    }

    public Map<Integer, Integer> getAdjacentDistances() {
        return adjacentDistances;
    }

    @Override
    public String toString(){
        String s = String.format("l %d %d %d %d", senderPort, originPort, sequenceNumber, timeToLive);
        for(Map.Entry<Integer, Integer> e : adjacentDistances.entrySet()){
            s += " " + e.getKey() + "-" + e.getValue();
        }
        return s;
    }

    public String toSenderString(int senderPort){
        String s = String.format("l %d %d %d %d", senderPort, this.originPort, this.sequenceNumber, this.timeToLive);
        for(Map.Entry<Integer, Integer> e : this.adjacentDistances.entrySet()){
            s += " " + e.getKey() + "-" + e.getValue();
        }
        return s;
    }
}
