import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Scanner;
import java.util.Map;

public class LinkStateSimulator {
    private static Map<Integer, MockRouter> map = new HashMap<>();

    public static void main(String[] args) {
        initializeRouters(args);

        listenToUserCommands();
    }

    private static void initializeRouters(String[] args){
        try (Scanner scanner = new Scanner(new File(args[0]))) {
            while (scanner.hasNextLine()) {
                String line = scanner.nextLine();
                String[] s = line.split(" ");
                int port = Integer.parseInt(s[0]);
                MockRouter mr = new MockRouter(port, Arrays.copyOfRange(s, 1, s.length));
                map.put(port, mr);
            }
        } catch (FileNotFoundException e) {
            System.out.println("Cannot find file.");
            System.exit(1);
        } catch (ArrayIndexOutOfBoundsException e) {
            System.out.println("Must include a topology file.");
            System.exit(1);
        }

        map.forEach((key, value) -> System.out.println(key + ":" + value));
        System.out.println("Initialization complete. Ready to accept commands.");
    }

    private static void listenToUserCommands(){
        Scanner s = new Scanner(System.in);
        while(true){
            String input = s.nextLine();

            if(input.length() == 0) continue;
            if(input.charAt(0) == 'e'){
                System.out.println("Goodbye!");
                System.exit(0);
            }else if (input.charAt(0) == 'h'){
                try {
                    onHCommand(Integer.parseInt(input.substring(2)));
                }catch (Exception e){
                    System.out.println("Use format: h some_port_number");
                }
            }else if (input.charAt(0) == 's'){
                try {
                    onSCommand(Integer.parseInt(input.substring(2)));
                }catch (Exception e){
                    System.out.println("Use format: s some_port_number");
                }
            }else{
                System.out.printf("Allowed Commands:\n" +
                        "h some_port_number #returns the result of sending an h message to the router some_port_number\n" +
                        "s some_port_number #shuts down MockRouter some_port_number\n" +
                        "e #exits the program\n");
            }
        }
    }

    private static void onHCommand(int portNumber){
        //System.out.printf("received h command for %d\n", portNumber);
        try {
            Socket socket = new Socket("localhost", portNumber);

            BufferedReader reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
            PrintWriter writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);

            writer.println("h");
            String line;
            while ((line = reader.readLine()) != null) {
                System.out.println(line);
            }
            reader.close();
            writer.close();
            socket.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static void onSCommand(int portNumber){
        //System.out.printf("received s command for %d\n", portNumber);
        try {
            Socket socket = new Socket("localhost", portNumber);

            BufferedReader reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
            PrintWriter writer = new PrintWriter(new BufferedWriter(new OutputStreamWriter(socket.getOutputStream())), true);

            writer.println("s");
            String line;
            while ((line = reader.readLine()) != null) {
                System.out.println(line);
            }
            reader.close();
            writer.close();
            socket.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
